# encoding: utf-8

# Nagstamon - Nagios status monitor for your desktop
# Copyright (C) 2008-2021 Henri Wahl <henri@nagstamon.de> et al.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

import urllib.request
import urllib.parse
import urllib.error
import socket
import sys
import re
import copy

from datetime import datetime, timedelta

from Nagstamon.Objects import *
from Nagstamon.Servers.Generic import GenericServer
from Nagstamon.Config import conf
from Nagstamon.Helpers import webbrowser_open


class CentreonServer(GenericServer):
    TYPE = 'Centreon'

    # centreon generic web interface uses a sid which is needed to ask for news
    SID = None

    # HARD/SOFT state mapping
    HARD_SOFT = {'(H)': 'hard', '(S)': 'soft'}

    # apparently necessesary because of non-english states as in https://github.com/HenriWahl/Nagstamon/issues/91 (Centeron 2.5)
    TRANSLATIONS = {'INDISPONIBLE': 'DOWN',
                    'INJOIGNABLE': 'UNREACHABLE',
                    'CRITIQUE': 'CRITICAL',
                    'INCONNU': 'UNKNOWN',
                    'ALERTE': 'WARNING'}

    # Entries for monitor default actions in context menu
    MENU_ACTIONS = ['Monitor', 'Recheck', 'Acknowledge', 'Downtime']

    # Centreon works better or at all with html.parser for BeautifulSoup
    PARSER = 'html.parser'

    # Needed to detect each Centreon's version
    centreon_version = None
    # Token that centreon use to protect the system
    centreon_token = None
    # To only detect broker once
    first_login = True
    # limit number of services retrived
    limit_services_number = 9999
    # default value, applies to version 2.2 and others
    XML_PATH = 'xml'

    def init_config(self):
        '''
        dummy init_config, called at thread start, not really needed here, just omit extra properties
        '''
        # set URLs here already
        self.init_HTTP()
        if not self.tls_error and self.centreon_version is not None:
            self._define_url()

    def init_HTTP(self):
        """
        initialize HTTP connection
        """
        if self.session is None:
            GenericServer.init_HTTP(self)

        if self.centreon_version is None:
            result_versioncheck = self.FetchURL(self.monitor_cgi_url + '/index.php', giveback='raw')
            raw_versioncheck, error_versioncheck = result_versioncheck.result, result_versioncheck.error
            if error_versioncheck == '':
                if re.search('2\.2\.[0-9]', raw_versioncheck):
                    self.centreon_version = 2.2
                    if conf.debug_mode is True:
                        self.Debug(server=self.get_name(), debug='Centreon version detected : 2.2')
                    # URLs for browser shortlinks/buttons on popup window
                    self.BROWSER_URLS = {'monitor': '$MONITOR$/main.php?p=1',
                                        'hosts': '$MONITOR$/main.php?p=20103&o=hpb',
                                        'services': '$MONITOR$/main.php?p=20202&o=svcpb',
                                        'history': '$MONITOR$/main.php?p=203'}
                elif re.search('2\.[3-6]\.[0-5]', raw_versioncheck):
                    self.centreon_version = 2.3456
                    if conf.debug_mode is True:
                        self.Debug(server=self.get_name(), debug='Centreon version detected : 2.3 <=> 2.6.5')
                    # URLs for browser shortlinks/buttons on popup window
                    self.BROWSER_URLS = {'monitor': '$MONITOR$/main.php?p=1',
                                        'hosts': '$MONITOR$/main.php?p=20103&o=hpb',
                                        'services': '$MONITOR$/main.php?p=20202&o=svcpb',
                                        'history': '$MONITOR$/main.php?p=203'}
                elif re.search('2\.6\.[6-9]', raw_versioncheck):
                    self.centreon_version = 2.66
                    if conf.debug_mode is True:
                        self.Debug(server=self.get_name(), debug='Centreon version detected : 2.6.6')
                    # URLs for browser shortlinks/buttons on popup window
                    self.BROWSER_URLS = {'monitor': '$MONITOR$/main.php?p=1',
                                        'hosts': '$MONITOR$/main.php?p=20103&o=hpb',
                                        'services': '$MONITOR$/main.php?p=20202&o=svcpb',
                                        'history': '$MONITOR$/main.php?p=203'}
                elif re.search('2\.7\.[0-9]', raw_versioncheck):
                    # Centreon 2.7 only support C. Broker
                    self.centreon_version = 2.7
                    if conf.debug_mode is True:
                        self.Debug(server=self.get_name(), debug='Centreon version detected : 2.7')
                    # URLs for browser shortlinks/buttons on popup window
                    self.BROWSER_URLS = {'monitor': '$MONITOR$/main.php?',
                                        'hosts': '$MONITOR$/main.php?p=20202&o=hpb',
                                        'services': '$MONITOR$/main.php?p=20201&o=svcpb',
                                        'history': '$MONITOR$/main.php?p=203'}
                elif re.search('2\.8\.[0-9]', raw_versioncheck):
                    # Centreon 2.8 only support C. Broker
                    self.centreon_version = 2.8
                    if conf.debug_mode is True:
                        self.Debug(server=self.get_name(), debug='Centreon version detected : 2.8')
                    # URLs for browser shortlinks/buttons on popup window
                    self.BROWSER_URLS = {'monitor': '$MONITOR$/main.php?',
                                        'hosts': '$MONITOR$/main.php?p=20202',
                                        'services': '$MONITOR$/main.php?p=20201',
                                        'history': '$MONITOR$/main.php?p=203'}
                elif re.search('18\.10\.[0-9]', raw_versioncheck):
                    self.centreon_version = 18.10
                    if conf.debug_mode is True:
                        self.Debug(server=self.get_name(), debug='Centreon version detected : 18.10')
                    # URLs for browser shortlinks/buttons on popup window
                    self.BROWSER_URLS = {'monitor': '$MONITOR$/main.php?',
                                        'hosts': '$MONITOR$/main.php?p=20202',
                                        'services': '$MONITOR$/main.php?p=20201',
                                        'history': '$MONITOR$/main.php?p=203'}
                elif re.search('19\.(04|10)\.[0-9]', raw_versioncheck) or re.search('20\.(04|10)\.[0-9]', raw_versioncheck):
                    self.centreon_version = 19.04
                    if conf.debug_mode is True:
                        self.Debug(server=self.get_name(), debug='Centreon version detected : 19.04 <=> 20.10')
                    # URLs for browser shortlinks/buttons on popup window
                    self.BROWSER_URLS = {'monitor': '$MONITOR$/main.php?',
                                        'hosts': '$MONITOR$/main.php?p=20202',
                                        'services': '$MONITOR$/main.php?p=20201',
                                        'history': '$MONITOR$/main.php?p=203'}
                else:
                    # unsupported version or unable do determine
                    self.centreon_version = 19.04
                    if conf.debug_mode is True:
                        self.Debug(server=self.get_name(), debug='Centreon version unknown : supposed to be >= 19.04')
                    # URLs for browser shortlinks/buttons on popup window
                    self.BROWSER_URLS = {'monitor': '$MONITOR$/main.php?',
                                        'hosts': '$MONITOR$/main.php?p=20202&o=hpb',
                                        'services': '$MONITOR$/main.php?p=20201&o=svcpb',
                                        'history': '$MONITOR$/main.php?p=203'}
            else:
                if conf.debug_mode is True:
                    self.Debug(server=self.get_name(), debug='Error getting the home page : ' + error_versioncheck)

            if self.first_login:
                self.SID = self._get_sid().result
                self.first_login = False

            del result_versioncheck, raw_versioncheck, error_versioncheck

    def reset_HTTP(self):
        '''
        Centreon needs deletion of SID
        '''
        self.SID = None
        self.SID = self._get_sid().result

    def open_monitor(self, host, service=''):
        if self.use_autologin is True:
            auth = '&autologin=1&useralias=' + self.username + '&token=' + self.autologin_key
        else:
            auth = ''

        #  Meta
        if host == '_Module_Meta':
            # Centreon < 2.7
            if self.centreon_version < 2.7:
                webbrowser_open(self.urls_centreon['index'] + '?' + urllib.parse.urlencode({'p': 20206, 'o': 'meta'}) + auth )
            #  Centreon 2.7
            elif self.centreon_version == 2.7:
                webbrowser_open(self.urls_centreon['main'] + '?' + urllib.parse.urlencode({'p':20206, 'o':'meta'}) + auth )
            #  Centreon 2.8
            elif self.centreon_version == 2.8:
                m =  re.search(r'^.+ \((?P<rsd>.+)\)$', service)
                if m:
                    service = m.group('rsd')
                    webbrowser_open(self.urls_centreon['main'] + '?' + urllib.parse.urlencode({'p':20201,'o':'svcd','host_name':'_Module_Meta','service_description':service}) + auth )
            # Starting from Centreon 18.10
            else:
                m =  re.search(r'^.+ \((?P<rsd>.+)\)$', service)
                if m:
                    service = m.group('rsd')
                    webbrowser_open(self.urls_centreon['main_with_frames'] + '?' + urllib.parse.urlencode({'p':20201,'o':'svcd','host_name':'_Module_Meta','service_description':service}) + auth )

        # must be a host if service is empty
        elif service == '':
            if self.centreon_version < 2.7:
                webbrowser_open(self.urls_centreon['main'] + '?' + urllib.parse.urlencode({'p':201,'o':'hd', 'host_name':host}) + auth )
            elif self.centreon_version in [2.7, 2.8]:
                webbrowser_open(self.urls_centreon['main'] + '?' + urllib.parse.urlencode({'p':20202,'o':'hd', 'host_name':host}) + auth )
            else:
                webbrowser_open(self.urls_centreon['main_with_frames'] + '?' + urllib.parse.urlencode({'p':20202,'o':'hd', 'host_name':host}) + auth )

        # so it's a service
        else:
            if self.centreon_version < 2.7:
                webbrowser_open(self.urls_centreon['main'] + '?' + urllib.parse.urlencode({'p':202, 'o':'svcd',  'host_name':host, 'service_description':service}) + auth )
            elif self.centreon_version in [2.7, 2.8]:
                webbrowser_open(self.urls_centreon['main'] + '?' + urllib.parse.urlencode({'p':20201,'o':'svcd', 'host_name':host, 'service_description':service}) + auth )
            else:
                webbrowser_open(self.urls_centreon['main_with_frames'] + '?' + urllib.parse.urlencode({'p':20201,'o':'svcd', 'host_name':host, 'service_description':service}) + auth )


    def _get_sid(self):
        '''
        gets a shiny new SID for XML HTTP requests to Centreon cutting it out via .partition() from raw HTML
        additionally get php session cookie
        '''
        try:
            # Aulogin with key, BROWSER_URLS needs the key
            if self.use_autologin == True:
                auth = '&autologin=1&useralias=' + self.username + '&token=' + self.autologin_key
                self.BROWSER_URLS= { 'monitor': self.BROWSER_URLS['monitor'] + auth,\
                                    'hosts': self.BROWSER_URLS['hosts'] + auth,\
                                    'services': self.BROWSER_URLS['services'] + auth,\
                                    'history': self.BROWSER_URLS['history'] + auth}
                raw = self.FetchURL(self.monitor_cgi_url + '/index.php?p=101&autologin=1&useralias=' + self.username + '&token=' + self.autologin_key, giveback='raw')
                if conf.debug_mode == True:
                    self.Debug(server=self.get_name(), debug='Autologin : ' + self.username + ' : ' + self.autologin_key)
                # Gathering of the token who will be used to interact with Centreon (start with 2.66)
                if  self.centreon_version >= 2.66 and self.centreon_version < 19.04:
                    page = self.FetchURL(self.monitor_cgi_url + '/main.get.php')
                    self.centreon_token = page.result.find('input', {'name': "centreon_token"})['value']

            # Password auth
            else:
                login = self.FetchURL(self.monitor_cgi_url + '/index.php')
                if login.error == '' and login.status_code == 200:
                    # Centreon >= 2.6.6 implement a token
                    if  self.centreon_version >= 2.66 and self.centreon_version <= 19.04:
                        form = login.result.find('form')
                        form_inputs = {}
                        # Need to catch the centreon_token for login to work
                        for form_input in ('centreon_token', 'submitLogin'):
                            form_inputs[form_input] = form.find('input', {'name': form_input})['value']
                        self.centreon_token = form_inputs['centreon_token']
                        form_inputs['useralias'] = self.username
                        form_inputs['password'] = self.password
                        # fire up login button with all needed data
                        raw = self.FetchURL(self.monitor_cgi_url + '/index.php', cgi_data=form_inputs)
                    else:
                        login_data = {"useralias" : self.username, "password" : self.password, "submit" : "Login"}
                        raw = self.FetchURL(self.monitor_cgi_url + "/index.php",cgi_data=login_data, giveback="raw")
                if conf.debug_mode == True:
                    self.Debug(server=self.get_name(), debug='Password login : ' + self.username + ' : ' + self.password)

            sid = self.session.cookies['PHPSESSID']
            if conf.debug_mode == True:
                self.Debug(server=self.get_name(), debug='SID : ' + sid)
                if  self.centreon_version >= 2.66 and self.centreon_version < 19.04:
                    self.Debug(server=self.get_name(), debug='Centreon Token : ' + self.centreon_token)
            # those broker urls would not be changing too often so this check migth be done here
            if self.first_login:
                self._get_xml_path(sid)
                self.first_login = False
            return Result(result=sid)

        except:
            import traceback
            traceback.print_exc(file=sys.stdout)
            result, error = self.Error(sys.exc_info())
            return Result(result=result, error=error)


    def get_start_end(self, host):
        '''
        get start and end time for downtime from Centreon server
        '''
        try:
            # It's not possible since 18.10 to get date from the webinterface
            # because it's set in javascript
            if self.centreon_version < 18.10:
                cgi_data = {'o':'ah',\
                            'host_name':host}
                if self.centreon_version < 2.7:
                    cgi_data['p'] = '20106'
                elif self.centreon_version == 2.7:
                    cgi_data['p'] = '210'
                elif self.centreon_version == 2.8:
                    cgi_data['o'] = 'a'
                    cgi_data['p'] = '210'
                result = self.FetchURL(self.urls_centreon['main'], cgi_data = cgi_data, giveback='obj')

                html, error = result.result, result.error
                if error == '':
                    start_date = html.find(attrs={'name':'start'}).attrs['value']
                    start_hour = html.find(attrs={'name':'start_time'}).attrs['value']
                    start_time = start_date + ' ' + start_hour

                    end_date = html.find(attrs={'name':'end'}).attrs['value']
                    end_hour = html.find(attrs={'name':'end_time'}).attrs['value']
                    end_time = end_date + ' ' + end_hour
                    return start_time, end_time

            else:
                start_time = datetime.now().strftime("%m/%d/%Y %H:%M")
                end_time = datetime.now() + timedelta(hours=2)
                end_time = end_time.strftime("%m/%d/%Y %H:%M")
                return start_time, end_time

        except:
            self.Error(sys.exc_info())
            return 'n/a', 'n/a'


    def GetHost(self, host):
        '''
        Centreonified way to get host ip - attribute 'a' in down hosts xml is of no use for up
        hosts so we need to get ip anyway from web page
        '''
        # the fastest method is taking hostname as used in monitor
        if conf.connect_by_host == True or host == '':
            return Result(result=host)

        # do a web interface search limited to only one result - the hostname
        cgi_data = {'sid': self.SID,
                    'search': host,
                    'num': 0,
                    'limit': 1,
                    'sort_type':'hostname',
                    'order': 'ASC',
                    'date_time_format_status': 'd/m/Y H:i:s',
                    'o': 'h',
                    'p': 20102,
                    'time': 0}

        centreon_hosts = self.urls_centreon['xml_hosts'] + '?' + urllib.parse.urlencode(cgi_data)

        result = self.FetchURL(centreon_hosts, giveback='xml')
        xmlobj, error, status_code = result.result, result.error, result.status_code

        # initialize ip string
        ip = ''

        if len(xmlobj) != 0:
            ip = str(xmlobj.l.a.text)
            # when connection by DNS is not configured do it by IP
            try:
                if conf.connect_by_dns == True:
                   # try to get DNS name for ip (reverse DNS), if not available use ip
                    try:
                        address = socket.gethostbyaddr(ip)[0]
                    except:
                        if conf.debug_mode == True:
                            self.Debug(server=self.get_name(), debug='Unable to do a reverse DNS lookup on IP: ' + ip)
                        address = ip
                else:
                    address = ip
            except:
                result, error = self.Error(sys.exc_info())
                return Result(result=result, error=error)

        else:
            result, error = self.Error(sys.exc_info())
            return Result(error=error)

        del xmlobj

        # print IP in debug mode
        if conf.debug_mode == True:
            self.Debug(server=self.get_name(), debug='IP of %s:' % (host) + ' ' + address)

        # give back host or ip
        return Result(result=address)


    def _get_xml_path(self, sid):
        '''
        Find out where this instance of Centreon is publishing the status XMLs
        Centreon 2.6 + ndo/c.broker - /include/monitoring/status/Hosts/xml/{ndo,broker}/hostXML.php according to configuration
        Centreon 2.7 + c.broker - /include/monitoring/status/Hosts/xml/hostXML.php
        Centreon 2.8 + c.broker - /include/monitoring/status/Hosts/xml/hostXML.php
        regexping HTML for Javascript
        '''
        if self.centreon_version <= 2.66:
            # 2.6 support NDO and C. Broker, we must check which one is used
            cgi_data = {'p':201, 'sid':sid}
            result = self.FetchURL(self.monitor_cgi_url + '/main.php', cgi_data=cgi_data, giveback='raw')
            raw, error = result.result, result.error
            if error == '':
                if re.search('var _addrXML.*xml\/ndo\/host', raw):
                  self.XML_PATH = 'xml/ndo'
                  if conf.debug_mode == True:
                      self.Debug(server=self.get_name(), debug='Detected broker : NDO')
                elif re.search('var _addrXML.*xml\/broker\/host', raw):
                    self.XML_PATH = 'xml/broker'
                    if conf.debug_mode == True:
                        self.Debug(server=self.get_name(), debug='Detected broker : C. Broker')
                else:
                    if conf.debug_mode == True:
                        self.Debug(server=self.get_name(), debug='Could not detect the broker for Centeron 2.[3-6]. Using Centreon Broker')
                    self.XML_PATH = 'xml/broker'
                del raw
            else:
                if conf.debug_mode == True:
                    self.Debug(server=self.get_name(), debug='Unable to fetch the main page to detect the broker : ' + error)
            del result, error
        else:
            if conf.debug_mode == True:
                self.Debug(server=self.get_name(), debug='Only Centreon Broker is supported in Centeon >= 2.7 -> XML_PATH='+ self.XML_PATH)


    def _define_url(self):
        urls_centreon_2_2 = {
            'main': self.monitor_cgi_url + '/main.php',
            'index': self.monitor_cgi_url + '/index.php',
            'xml_services': self.monitor_cgi_url + '/include/monitoring/status/Services/' + self.XML_PATH + '/serviceXML.php',
            'xml_hosts': self.monitor_cgi_url + '/include/monitoring/status/Hosts/' + self.XML_PATH + '/hostXML.php',
            'xml_meta': self.monitor_cgi_url + '/include/monitoring/status/Meta/' + self.XML_PATH + '/metaServiceXML.php',
            'xml_hostSendCommand': self.monitor_cgi_url + '/include/monitoring/objectDetails/xml/hostSendCommand.php',
            'xml_serviceSendCommand': self.monitor_cgi_url + '/include/monitoring/objectDetails/xml/serviceSendCommand.php',
            'external_cmd_cmdPopup': self.monitor_cgi_url + '/include/monitoring/external_cmd/cmdPopup.php',
            # no idea if this really exist in centreon < 2.7
            'autologoutXMLresponse': self.monitor_cgi_url + '/include/common/javascript/autologoutXMLresponse.php'
        }

        # inconsistant url in Centreon 2.7
        urls_centreon_2_7 = {
            'main': self.monitor_cgi_url + '/main.php',
            'index': self.monitor_cgi_url + '/index.php',
            'xml_services': self.monitor_cgi_url + '/include/monitoring/status/Services/' + self.XML_PATH + '/serviceXML.php',
            'xml_hosts': self.monitor_cgi_url + '/include/monitoring/status/Hosts/' + self.XML_PATH + '/broker/hostXML.php',
            'xml_meta': self.monitor_cgi_url + '/include/monitoring/status/Meta/' + self.XML_PATH + '/broker/metaServiceXML.php',
            'xml_hostSendCommand': self.monitor_cgi_url + '/include/monitoring/objectDetails/xml/hostSendCommand.php',
            'xml_serviceSendCommand': self.monitor_cgi_url + '/include/monitoring/objectDetails/xml/serviceSendCommand.php',
            'external_cmd_cmdPopup': self.monitor_cgi_url + '/include/monitoring/external_cmd/cmdPopup.php',
            'autologoutXMLresponse': self.monitor_cgi_url + '/include/common/javascript/autologoutXMLresponse.php'
        }

        urls_centreon_2_8 = {
            'main': self.monitor_cgi_url + '/main.php',
            'index': self.monitor_cgi_url + '/index.php',
            'xml_services': self.monitor_cgi_url + '/include/monitoring/status/Services/' + self.XML_PATH + '/serviceXML.php',
            'xml_hosts': self.monitor_cgi_url + '/include/monitoring/status/Hosts/' + self.XML_PATH + '/hostXML.php',
            'xml_hostSendCommand': self.monitor_cgi_url + '/include/monitoring/objectDetails/xml/hostSendCommand.php',
            'xml_serviceSendCommand': self.monitor_cgi_url + '/include/monitoring/objectDetails/xml/serviceSendCommand.php',
            'external_cmd_cmdPopup': self.monitor_cgi_url + '/include/monitoring/external_cmd/cmdPopup.php',
            'autologoutXMLresponse': self.monitor_cgi_url + '/include/core/autologout/autologoutXMLresponse.php'
        }

        urls_centreon_18_10 = {
            'main': self.monitor_cgi_url + '/main.get.php',
            # needed to get the frames around the page when opening the monitoring on a host/service
            'main_with_frames': self.monitor_cgi_url + '/main.php',
            'index': self.monitor_cgi_url + '/index.php',
            'xml_services': self.monitor_cgi_url + '/include/monitoring/status/Services/' + self.XML_PATH + '/serviceXML.php',
            'xml_hosts': self.monitor_cgi_url + '/include/monitoring/status/Hosts/' + self.XML_PATH + '/hostXML.php',
            'xml_hostSendCommand': self.monitor_cgi_url + '/include/monitoring/objectDetails/xml/hostSendCommand.php',
            'xml_serviceSendCommand': self.monitor_cgi_url + '/include/monitoring/objectDetails/xml/serviceSendCommand.php',
            'external_cmd_cmdPopup': self.monitor_cgi_url + '/include/monitoring/external_cmd/cmdPopup.php',
            'keepAlive': self.monitor_cgi_url + '/api/internal.php?object=centreon_keepalive&action=keepAlive'
        }

        if self.centreon_version < 2.7:
            self.urls_centreon = urls_centreon_2_2
        elif self.centreon_version == 2.7:
            self.urls_centreon = urls_centreon_2_7
        elif self.centreon_version == 2.8:
            self.urls_centreon = urls_centreon_2_8
        # 18.10 and beyond
        elif self.centreon_version >= 18.10:
            self.urls_centreon = urls_centreon_18_10
        if conf.debug_mode == True:
            self.Debug(server=self.get_name(), debug='URLs defined for Centreon %s' % (self.centreon_version))


    def _get_host_id(self, host):
        '''
        get host_id via parsing raw html
        '''
        if self.centreon_version < 2.7:
            cgi_data = {'p': 20102, 'o': 'hd', 'host_name': host, 'sid': self.SID}
        else:
            cgi_data = {'p': 20202, 'o': 'hd', 'host_name': host, 'sid': self.SID}

        url = self.urls_centreon['main'] + '?' + urllib.parse.urlencode(cgi_data)

        result = self.FetchURL(url, giveback='raw')
        raw, error = result.result, result.error

        if error == '':
            host_id = raw.partition("var host_id = '")[2].partition("'")[0]
            del raw
        else:
            if conf.debug_mode == True:
                self.Debug(server=self.get_name(), debug='Host ID could not be retrieved.')

        # some cleanup
        del result, error

        # only if host_id is an usable integer return it
        try:
            if int(host_id):
                if conf.debug_mode == True:
                    self.Debug(server=self.get_name(), host=host, debug='Host ID is ' + host_id)
                return host_id
            else:
                return ''
        except:
            return ''


    def _get_host_and_service_id(self, host, service):
        '''
        parse a ton of html to get a host and a service id...
        '''
        cgi_data = {'p':'20201',\
                    'host_name':host,\
                    'service_description':service,\
                    'o':'svcd'}

        # This request must be done in a GET, so just encode the parameters and fetch
        result = self.FetchURL(self.urls_centreon['main'] + '?' + urllib.parse.urlencode(cgi_data), giveback="raw")
        raw, error = result.result, result.error

        if error == '':
            host_id = raw.partition("var host_id = '")[2].partition("'")[0]
            svc_id = raw.partition("var svc_id = '")[2].partition("'")[0]
            del raw
            if conf.debug_mode == True:
                self.Debug(server=self.get_name(), host=host, service=service, debug='- Get host/svc ID : ' + host_id + '/' + svc_id)
        else:
            if conf.debug_mode == True:
                self.Debug(server=self.get_name(), host=host, service=service, debug='- IDs could not be retrieved.')

        # some cleanup
        del result, error

        # only if host_id is an usable integer return it
        try:
            if int(host_id) and int(svc_id):
                if conf.debug_mode == True:
                    self.Debug(server=self.get_name(), host=host, service=service, debug='- Host & Service ID are valid (int)')
                return host_id,svc_id
            else:
                return '',''
        except:
            return '',''


    def _get_status(self):
        '''
        Get status from Centreon Server
        '''
        # Be sure that the session is still active
        result = self._check_session()
        if result is not None:
            if result.result == 'ERROR':
                if 'urls_centreon' in result.error:
                    result.error = 'Connection error'
                return result

        # services (unknown, warning or critical?)
        if self.centreon_version < 2.7:
            nagcgiurl_services = self.urls_centreon['xml_services'] + '?' + urllib.parse.urlencode({'num':0, 'limit':self.limit_services_number, 'o':'svcpb', 'sort_type':'status', 'sid':self.SID})
        else:
            nagcgiurl_services = self.urls_centreon['xml_services'] + '?' + urllib.parse.urlencode({'num':0, 'limit':self.limit_services_number, 'o':'svcpb', 'p':20201, 'nc':0, 'criticality':0, 'statusService':'svcpb', 'sSetOrderInMemory':1, 'sid':self.SID})

        # hosts (up or down or unreachable)
        # define hosts xml URL, because of inconsistant url
        if self.centreon_version < 2.7:
            nagcgiurl_hosts = self.urls_centreon['xml_hosts'] + '?' + urllib.parse.urlencode({'num':0, 'limit':self.limit_services_number, 'o':'hpb', 'sort_type':'status', 'sid':self.SID})
        elif self.centreon_version >= 2.7 and self.centreon_version < 19.04:
            nagcgiurl_hosts = self.urls_centreon['xml_hosts'] + '?' + urllib.parse.urlencode({'num':0, 'limit':self.limit_services_number, 'o':'hpb', 'p':20202, 'criticality':0, 'statusHost':'hpb', 'sSetOrderInMemory':1, 'sid':self.SID})
        else:
            nagcgiurl_hosts = self.urls_centreon['xml_hosts'] + '?' + urllib.parse.urlencode({'num':0, 'limit':self.limit_services_number, 'o':'hpb', 'p':20202, 'criticality':0, 'statusHost':'hpb', 'sSetOrderInMemory':1})

        # hosts - mostly the down ones
        # unfortunately the hosts status page has a different structure so
        # hosts must be analyzed separately
        try:
            result = self.FetchURL(nagcgiurl_hosts, giveback='xml')
            xmlobj, error, status_code = result.result, result.error, result.status_code

            # check if any error occured
            errors_occured = self.check_for_error(xmlobj, error, status_code)

            # if there are errors return them
            if errors_occured != False:
                return(errors_occured)

            # Check if the result is not empty
            if len(xmlobj) == 0:
                if conf.debug_mode == True:
                    self.Debug(server=self.get_name(), debug='Empty host XML result')
                return Result(result=None, error="Empty host XML result")

            # in case there are no children session ID is expired
            if xmlobj.text.lower() == 'bad session id':
                del xmlobj
                if conf.debug_mode == True:
                    self.Debug(server=self.get_name(), debug='Bad session ID, retrieving new one...')

                # try again...
                self.SID = self._get_sid().result
                result = self.FetchURL(nagcgiurl_hosts, giveback='xml')
                xmlobj, error, status_code = result.result, result.error, result.status_code
                errors_occured = self.check_for_error(xmlobj, error, status_code)
                # if there are errors return them
                if errors_occured != False:
                    return(errors_occured)

                # a second time a bad session id should raise an error
                if xmlobj.text.lower() == 'bad session id':
                    if conf.debug_mode == True:
                        self.Debug(server=self.get_name(), debug='Even after renewing session ID, unable to get the XML')
                    return Result(result='ERROR',
                                  error='Bad session ID',
                                  status_code=status_code)

            for l in xmlobj.findAll('l'):
                try:
                    # host objects contain service objects
                    if not l.hn.text in self.new_hosts:
                        self.new_hosts[str(l.hn.text)] = GenericHost()
                        self.new_hosts[str(l.hn.text)].name =  str(l.hn.text)
                        self.new_hosts[str(l.hn.text)].server = self.name
                        self.new_hosts[str(l.hn.text)].status = str(l.cs.text)
                        # disgusting workaround for https://github.com/HenriWahl/Nagstamon/issues/91
                        if self.new_hosts[str(l.hn.text)].status in self.TRANSLATIONS:
                            self.new_hosts[str(l.hn.text)].status = self.TRANSLATIONS[self.new_hosts[str(l.hn.text)].status]
                        self.new_hosts[str(l.hn.text)].attempt, self.new_hosts[str(l.hn.text)].status_type  = str(l.tr.text).split(' ')
                        self.new_hosts[str(l.hn.text)].status_type = self.HARD_SOFT[self.new_hosts[str(l.hn.text)].status_type]
                        self.new_hosts[str(l.hn.text)].last_check = str(l.lc.text)
                        self.new_hosts[str(l.hn.text)].duration = str(l.lsc.text)
                        self.new_hosts[str(l.hn.text)].status_information= str(l.ou.text)
                        if l.find('cih') != None:
                            self.new_hosts[str(l.hn.text)].criticality = str(l.cih.text)
                        else:
                            self.new_hosts[str(l.hn.text)].criticality = ''
                        self.new_hosts[str(l.hn.text)].acknowledged = bool(int(str(l.ha.text)))
                        self.new_hosts[str(l.hn.text)].scheduled_downtime = bool(int(str(l.hdtm.text)))
                        if l.find('is') != None:
                            self.new_hosts[str(l.hn.text)].flapping = bool(int(str(l.find('is').text)))
                        else:
                            self.new_hosts[str(l.hn.text)].flapping = False
                        self.new_hosts[str(l.hn.text)].notifications_disabled = not bool(int(str(l.ne.text)))
                        self.new_hosts[str(l.hn.text)].passiveonly = not bool(int(str(l.ace.text)))
                except:
                    import traceback
                    traceback.print_exc(file=sys.stdout)
                    # set checking flag back to False
                    self.isChecking = False
                    result, error = self.Error(sys.exc_info())
                    return Result(result=result, error=error)

            del xmlobj

        except:
            import traceback
            traceback.print_exc(file=sys.stdout)
            # set checking flag back to False
            self.isChecking = False
            result, error = self.Error(sys.exc_info())
            return Result(result=result, error=error)

        # services
        try:
            result = self.FetchURL(nagcgiurl_services, giveback='xml')
            xmlobj, error, status_code = result.result, result.error, result.status_code

            # check if any error occured
            errors_occured = self.check_for_error(xmlobj, error, status_code)
            # if there are errors return them
            if errors_occured != False:
                return(errors_occured)

            # Check if the result is not empty
            if len(xmlobj) == 0:
                if conf.debug_mode == True:
                    self.Debug(server=self.get_name(), debug='Empty service XML result')
                return Result(result=None, error="Empty service XML result")

            # in case there are no children session id is invalid
            if xmlobj.text.lower() == 'bad session id':
                # debug
                if conf.debug_mode == True:
                    self.Debug(server=self.get_name(), debug='Bad session ID, retrieving new one...')
                # try again...
                self.SID = self._get_sid().result
                result = self.FetchURL(nagcgiurl_services, giveback='xml')
                xmlobj, error, status_code = result.result, result.error, result.status_code
                errors_occured = self.check_for_error(xmlobj, error, status_code)
                # if there are errors return them
                if errors_occured != False:
                    return(errors_occured)

                # a second time a bad session id should raise an error
                if xmlobj.text.lower() == 'bad session id':
                    return Result(result='ERROR',
                                  error='Bad session ID',
                                  status_code=status_code)

            # In Centreon 2.8, Meta are merged with regular services
            if self.centreon_version < 2.8:
                # define meta-services xml URL
                if self.centreon_version == 2.7:
                    nagcgiurl_meta_services = self.urls_centreon['xml_meta'] + '?' + urllib.parse.urlencode({'num':0, 'limit':self.limit_services_number, 'o':'meta', 'sort_type':'status', 'sid':self.SID})
                else:
                    nagcgiurl_meta_services = self.urls_centreon['xml_meta'] + '?' + urllib.parse.urlencode({'num':0, 'limit':self.limit_services_number, 'o':'meta', 'sort_type':'status', 'sid':self.SID})

                # retrive meta-services xml STATUS
                result_meta = self.FetchURL(nagcgiurl_meta_services, giveback='xml')
                xmlobj_meta, error_meta, status_code_meta = result_meta.result, result_meta.error, result_meta.status_code

                # check if any error occured
                errors_occured = self.check_for_error(xmlobj_meta, error_meta, status_code_meta)

                # if there are errors return them
                if errors_occured != False:
                    return(errors_occured)

                # a second time a bad session id should raise an error
                if xmlobj_meta.text.lower() == 'bad session id':
                    if conf.debug_mode == True:
                        self.Debug(server=self.get_name(), debug='Even after renewing session ID, unable to get the XML')

                    return Result(result='ERROR',
                                  error='Bad session ID',
                                  status_code=status_code_meta)

                # INSERT META-services xml at the end of the services xml
                try:
                        xmlobj.append(xmlobj_meta.reponse)
                except:
                        import traceback
                        traceback.print_exc(file=sys.stdout)
                        # set checking flag back to False
                        self.isChecking = False
                        result, error = self.Error(sys.exc_info())
                        return Result(result=result, error=error)
                # do some cleanup
                del xmlobj_meta

            for l in xmlobj.findAll('l'):
                try:
                    # host objects contain service objects
                    ###if not self.new_hosts.has_key(str(l.hn.text)):
                    if not l.hn.text in self.new_hosts:
                        self.new_hosts[str(l.hn.text)] = GenericHost()
                        self.new_hosts[str(l.hn.text)].name = str(l.hn.text)
                        self.new_hosts[str(l.hn.text)].status = 'UP'
                    # if a service does not exist create its object
                    if not l.sd.text in self.new_hosts[str(l.hn.text)].services:
                        self.new_hosts[str(l.hn.text)].services[str(l.sd.text)] = GenericService()
                        self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].host = str(l.hn.text)
                        self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].name = str(l.sd.text)
                        self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].server = self.name
                        self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status = str(l.cs.text)

                        if self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].host == '_Module_Meta':
                            # ajusting service name for Meta services
                            if self.centreon_version < 2.8:
                                self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].name = '{} ({})'.format(str(l.sd.text), l.rsd.text)
                                self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].attempt = str(l.ca.text)
                            else:
                                self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].name = '{} ({})'.format(str(l.sdn.text), l.sdl.text)
                                self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].attempt, \
                                self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status_type = str(l.ca.text).split(' ')
                        else:
                            self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].attempt, \
                            self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status_type = str(l.ca.text).split(' ')

                        # disgusting workaround for https://github.com/HenriWahl/Nagstamon/issues/91
                        # Still needed in Centreon 2.8 at least : https://github.com/HenriWahl/Nagstamon/issues/344
                        # Need enhancement, we can do service state matching with this field <sc>service_unknown</sc>
                        #if self.centreon_version < 2.66:
                        if self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status in self.TRANSLATIONS:
                            self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status = self.TRANSLATIONS[\
                            self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status]

                        if not (self.centreon_version < 2.8 and self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].host == '_Module_Meta'):
                            self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status_type =\
                            self.HARD_SOFT[self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status_type]

                        if conf.debug_mode == True:
                            self.Debug(server=self.get_name(), debug='Parsing service XML (Host/Service/Status_type) ' + self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].host + '/' + self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].name + '/' + self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status_type)
                        self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].last_check = str(l.lc.text)
                        self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].duration = str(l.d.text)
                        self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status_information = str(l.po.text).replace('\n', ' ').strip()

                        if l.find('cih') != None:
                            self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].criticality = str(l.cih.text)
                        else:
                            self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].criticality = ''

                        self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].acknowledged = bool(int(str(l.pa.text)))
                        self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].notifications_disabled = not bool(int(str(l.ne.text)))

                        # for features not available in centreon < 2.8 and meta services
                        if not (self.centreon_version < 2.8 and self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].host == '_Module_Meta'):
                            self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].scheduled_downtime = bool(int(str(l.dtm.text)))
                            self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].flapping = bool(int(str(l.find('is').text)))
                            self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].passiveonly = not bool(int(str(l.ac.text)))

                except:
                    import traceback
                    traceback.print_exc(file=sys.stdout)
                    # set checking flag back to False
                    self.isChecking = False
                    result, error = self.Error(sys.exc_info())
                    return Result(result=result, error=error)

            # do some cleanup
            del xmlobj

        except:
            import traceback
            traceback.print_exc(file=sys.stdout)
            # set checking flag back to False
            self.isChecking = False
            result, error = self.Error(sys.exc_info())
            return Result(result=result, error=error)

        # return True if all worked well
        return Result()


    def _set_acknowledge(self, host, service, author, comment, sticky, notify, persistent, all_services=[]):
        # decision about host or service - they have different URLs
        try:
            if service == '':
                # host
                cgi_data = {'cmd': '14',
                            'host_name': host,
                            'author': author,
                            'comment': comment,
                            'submit': 'Add',
                            'notify': int(notify),
                            'persistent': int(persistent),
                            'sticky': int(sticky),
                            'ackhostservice': '0',
                            'en': '1'}
                if self.centreon_version < 2.7:
                    cgi_data['p'] = '20105'
                    cgi_data['o'] = 'hpb'
                else:
                    cgi_data['p'] = '20202'
                    cgi_data['o'] = 'hpb'
                    cgi_data['centreon_token'] = self.centreon_token

                # Post
                raw = self.FetchURL(self.urls_centreon['main'], cgi_data=cgi_data, giveback='raw')
                del raw

            # if host is acknowledged and all services should be to or if a service is acknowledged
            # (and all other on this host too)
            if service != '' or len(all_services) > 0:
                # service(s) @ host
                # if all_services is empty only one service has to be checked - the one clicked
                # otherwise if there all services should be acknowledged
                if len(all_services) == 0: all_services = [service]

                # acknowledge all services on a host
                for s in all_services:
                    cgi_data = {'cmd': '15',
                                'host_name': host,
                                'author': author,
                                'comment': comment,
                                'submit': 'Add',
                                'notify': int(notify),
                                'service_description': s,
                                'force_check': '1',
                                'persistent': int(persistent),
                                # following not needed in 18.10, required in wich version ?
                                'persistant': int(persistent),
                                'sticky': int(sticky),
                                'o': 'svcd',
                                'en': '1'}
                    if self.centreon_version < 2.7:
                        cgi_data['p'] = '20215'
                    else:
                        cgi_data['p'] = '20201'
                        cgi_data['centreon_token'] = self.centreon_token

                    # in case of a meta-service, extract the 'rsd' field from the service name :
                    if host == '_Module_Meta':
                        m =  re.search(r'^.+ \((?P<rsd>.+)\)$', s)
                        if m:
                            rsd = m.group('rsd')
                            if self.centreon_version < 2.8:
                                cgi_data = {'p': '20206',
                                            'o': 'meta',
                                            'cmd': '70',
                                            'select[' + host + ';' + rsd + ']': '1',
                                            'limit': '0'}
                            elif self.centreon_version in [2.8, 18.10]:
                                cgi_data['service_description'] = rsd

                    # POST, for some strange reason only working if giveback is 'raw'
                    raw = self.FetchURL(self.urls_centreon['main'], cgi_data=cgi_data, giveback='raw')
                    del raw
        except:
            self.Error(sys.exc_info())


    def _set_recheck(self, host, service):
        '''
        host and service ids are needed to tell Centreon what whe want
        '''
        try:
        # decision about host or service - they have different URLs
            #  Meta
            if host == '_Module_Meta':
                if conf.debug_mode == True:
                    self.Debug(server=self.get_name(), debug='Recheck on a Meta service, more work to be done')
                m =  re.search(r'^.+ \((?P<rsd>.+)\)$', service)
                if m:
                    rsd = m.group('rsd')
                    if self.centreon_version < 2.8:
                        url = self.urls_centreon['main'] + '?' + urllib.parse.urlencode({'p': '20206','o': 'meta','cmd': '3','select[' + host + ';' + rsd + ']': '1','limit':'0'})
                    else:
                        url = self.urls_centreon['main'] + '?' + urllib.parse.urlencode({'p': '202','o': 'svc','cmd': '3','select[' + host + ';' + rsd + ']': '1','limit':'1','centreon_token':self.centreon_token})

            elif service == '':
                # ... it can only be a host, so check all his services and there is a command for that
                host_id = self._get_host_id(host)

                if self.centreon_version < 2.7:
                    url = self.urls_centreon['xml_hostSendCommand'] + '?' + urllib.parse.urlencode({'cmd':'host_schedule_check', 'actiontype':1,'host_id':host_id,'sid':self.SID})
                else:
                    url = self.urls_centreon['xml_hostSendCommand'] + '?' + urllib.parse.urlencode({'cmd':'host_schedule_check', 'actiontype':1,'host_id':host_id})
                del host_id

            else:
                # service @ host
                host_id, service_id = self._get_host_and_service_id(host, service)

                # Starting from 19.04 this must be in POST
                if self.centreon_version < 19.04:
                    # fill and encode URL data
                    cgi_data = urllib.parse.urlencode({'cmd':'service_schedule_check', 'actiontype':1,\
                                                 'host_id':host_id, 'service_id':service_id, 'sid':self.SID})

                    url = self.urls_centreon['xml_serviceSendCommand'] + '?' + cgi_data
                    del host_id, service_id
                else:
                    cgi_data = {'cmd': 'service_schedule_check',
                                'host_id': host_id,
                                'service_id': service_id,
                                'actiontype': '0'}
                    del host_id, service_id

            if self.centreon_version < 19.04:
                # execute GET request
                raw = self.FetchURL(url, giveback='raw')
                del raw
            else:
                # running remote cgi command with POST method, for some strange reason only working if
                # giveback is 'raw'
                raw = self.FetchURL(self.urls_centreon['xml_serviceSendCommand'], cgi_data=cgi_data, giveback='raw')
                del raw
        except:
            self.Error(sys.exc_info())


    def _set_downtime(self, host, service, author, comment, fixed, start_time, end_time, hours, minutes):
        '''
        gets actual host and service ids and apply them to downtime cgi
        '''
        try:
            # duration unit is minute
            duration = (hours * 60) + minutes
            # need cmdPopup.php needs boolean
            if fixed == 1:
                fixed = 'true'
            else:
                fixed = 'false'

            # Host downtime
            if service == '':
                if self.centreon_version < 19.04:
                    cgi_data = {'cmd':75,\
                                'duration':duration,\
                                'duration_scale':'m',\
                                'start':start_time,\
                                'end':end_time,\
                                'comment':comment,\
                                'fixed':fixed,\
                                'downtimehostservice':'true',\
                                'author':author,\
                                'sid':self.SID,\
                                'select['+host+']':1
                                }
                # Params has changed starting from 19.04
                else:
                    cgi_data = {'cmd':75,
                                'duration':duration,
                                'duration_scale':'m',
                                'comment':comment,
                                'start':start_time,
                                'end':end_time,
                                'host_or_centreon_time':0,
                                'fixed':fixed,
                                'downtimehostservice':'true',
                                'author':author,
                                'resources':'["'+host+'"]'
                                }

            # Service downtime
            else:
                # Centreon 2.8 only, in case of a meta-service, extract the 'rsd' field from the service name :
                if host == '_Module_Meta' and self.centreon_version in [2.8, 18.10]:
                    m =  re.search(r'^.+ \((?P<rsd>.+)\)$', service)
                    if m:
                        rsd = m.group('rsd')
                        service = rsd
                if self.centreon_version < 19.04:
                    cgi_data = {'cmd':74,\
                                'duration':duration,\
                                'duration_scale':'m',\
                                'start':start_time,\
                                'end':end_time,\
                                'comment':comment,\
                                'fixed':fixed,\
                                'downtimehostservice':0,\
                                'author':author,\
                                'sid':self.SID,\
                                'select['+host+';'+service+']':1
                                }

                # Params has changed starting from 19.04
                else:
                    cgi_data = {'cmd':74,
                                'duration':duration,
                                'duration_scale':'m',
                                'comment':comment,
                                'start':start_time,
                                'end':end_time,
                                'host_or_centreon_time':0,
                                'fixed':fixed,
                                'downtimehostservice':0,
                                'author':author,
                                'resources':'["'+host+'%3B'+service+'"]'
                                }

            if self.centreon_version < 19.04:
                # This request must be done in a GET, so just encode the parameters and fetch
                raw = self.FetchURL(self.urls_centreon['external_cmd_cmdPopup'] + '?' + urllib.parse.urlencode(cgi_data), giveback="raw")
                del raw
            # Starting from 19.04, must be POST
            else:
                # Do it in POST
                raw = self.FetchURL(self.urls_centreon['external_cmd_cmdPopup'], cgi_data=cgi_data, giveback='raw')
                del raw

        except:
            self.Error(sys.exc_info())


    def _check_session(self):
        if conf.debug_mode == True:
            self.Debug(server=self.get_name(), debug='Checking session status')
        if 'url_centreon' not in self.__dict__:
            self.init_config()
        try:
            if self.centreon_version >= 18.10:
                result = self.FetchURL(self.urls_centreon['keepAlive'], giveback='raw')
                self.raw, self.error, self.status_code = result.result, result.error, result.status_code
                # Return 200 & null a session is open
                if conf.debug_mode == True:
                    self.Debug(server=self.get_name(), debug='Session status : ' + self.raw + ', http code : ' + str(self.status_code))
                # 401 if no valid session is present
                if self.status_code == 401:
                    self.SID = self._get_sid().result
                    if conf.debug_mode == True:
                        self.Debug(server=self.get_name(), debug='Session renewed')

            else:
                result = self.FetchURL(self.urls_centreon['autologoutXMLresponse'], giveback='xml')
                xmlobj, error, status_code = result.result, result.error, result.status_code
                self.session_state = xmlobj.find("state").text.lower()
                if conf.debug_mode == True:
                    self.Debug(server=self.get_name(), debug='Session status : ' + self.session_state)
                if self.session_state == "nok":
                    self.SID = self._get_sid().result
                    if conf.debug_mode == True:
                        self.Debug(server=self.get_name(), debug='Session renewed')

        except:
            import traceback
            traceback.print_exc(file=sys.stdout)
            result, error = self.Error(sys.exc_info())
            return Result(result=result, error=error)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'dialog_about.ui'
#
# Created by: PyQt5 UI code generator 5.5.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_dialog_about(object):
    def setupUi(self, dialog_about):
        dialog_about.setObjectName("dialog_about")
        dialog_about.resize(500, 300)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(dialog_about.sizePolicy().hasHeightForWidth())
        dialog_about.setSizePolicy(sizePolicy)
        dialog_about.setSizeGripEnabled(True)
        dialog_about.setModal(True)
        self.gridLayout = QtWidgets.QGridLayout(dialog_about)
        self.gridLayout.setObjectName("gridLayout")
        self.tabs = QtWidgets.QTabWidget(dialog_about)
        self.tabs.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.tabs.setObjectName("tabs")
        self.tab_about = QtWidgets.QWidget()
        self.tab_about.setObjectName("tab_about")
        self.vbox_about = QtWidgets.QVBoxLayout(self.tab_about)
        self.vbox_about.setObjectName("vbox_about")
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.vbox_about.addItem(spacerItem)
        self.label_nagstamon = QtWidgets.QLabel(self.tab_about)
        self.label_nagstamon.setAlignment(QtCore.Qt.AlignCenter)
        self.label_nagstamon.setObjectName("label_nagstamon")
        self.vbox_about.addWidget(self.label_nagstamon)
        self.label_nagstamon_long = QtWidgets.QLabel(self.tab_about)
        self.label_nagstamon_long.setAlignment(QtCore.Qt.AlignCenter)
        self.label_nagstamon_long.setObjectName("label_nagstamon_long")
        self.vbox_about.addWidget(self.label_nagstamon_long)
        self.label_copyright = QtWidgets.QLabel(self.tab_about)
        self.label_copyright.setAlignment(QtCore.Qt.AlignCenter)
        self.label_copyright.setObjectName("label_copyright")
        self.vbox_about.addWidget(self.label_copyright)
        self.label_website = QtWidgets.QLabel(self.tab_about)
        self.label_website.setAlignment(QtCore.Qt.AlignCenter)
        self.label_website.setObjectName("label_website")
        self.vbox_about.addWidget(self.label_website)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.vbox_about.addItem(spacerItem1)
        self.label_footnote = QtWidgets.QLabel(self.tab_about)
        self.label_footnote.setAlignment(QtCore.Qt.AlignCenter)
        self.label_footnote.setObjectName("label_footnote")
        self.vbox_about.addWidget(self.label_footnote)
        self.tabs.addTab(self.tab_about, "")
        self.tab_credits = QtWidgets.QWidget()
        self.tab_credits.setObjectName("tab_credits")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.tab_credits)
        self.verticalLayout.setObjectName("verticalLayout")
        self.textedit_credits = QtWidgets.QTextBrowser(self.tab_credits)
        self.textedit_credits.setObjectName("textedit_credits")
        self.verticalLayout.addWidget(self.textedit_credits)
        self.tabs.addTab(self.tab_credits, "")
        self.tab_license = QtWidgets.QWidget()
        self.tab_license.setObjectName("tab_license")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.tab_license)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.textedit_license = QtWidgets.QTextEdit(self.tab_license)
        self.textedit_license.setObjectName("textedit_license")
        self.verticalLayout_3.addWidget(self.textedit_license)
        self.tabs.addTab(self.tab_license, "")
        self.gridLayout.addWidget(self.tabs, 0, 0, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(dialog_about)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 1, 0, 1, 1)

        self.retranslateUi(dialog_about)
        self.tabs.setCurrentIndex(0)
        self.buttonBox.accepted.connect(dialog_about.accept)
        self.buttonBox.rejected.connect(dialog_about.reject)
        QtCore.QMetaObject.connectSlotsByName(dialog_about)

    def retranslateUi(self, dialog_about):
        _translate = QtCore.QCoreApplication.translate
        dialog_about.setWindowTitle(_translate("dialog_about", "About Nagstamon"))
        self.label_nagstamon.setText(_translate("dialog_about", "Nagstamon x"))
        self.label_nagstamon_long.setText(_translate("dialog_about", "Nagios status monitor"))
        self.label_copyright.setText(_translate("dialog_about", "©2008-2021 Henri Wahl et al."))
        self.label_website.setText(_translate("dialog_about", "https://nagstamon.de"))
        self.label_footnote.setText(_translate("dialog_about", "Footnote"))
        self.tabs.setTabText(self.tabs.indexOf(self.tab_about), _translate("dialog_about", "About"))
        self.tabs.setTabText(self.tabs.indexOf(self.tab_credits), _translate("dialog_about", "Credits"))
        self.tabs.setTabText(self.tabs.indexOf(self.tab_license), _translate("dialog_about", "License"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'dialog_authentication.ui'
#
# Created by: PyQt5 UI code generator 5.5.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_dialog_authentication(object):
    def setupUi(self, dialog_authentication):
        dialog_authentication.setObjectName("dialog_authentication")
        dialog_authentication.resize(350, 226)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(dialog_authentication.sizePolicy().hasHeightForWidth())
        dialog_authentication.setSizePolicy(sizePolicy)
        dialog_authentication.setMinimumSize(QtCore.QSize(350, 0))
        dialog_authentication.setModal(True)
        self.gridLayout = QtWidgets.QGridLayout(dialog_authentication)
        self.gridLayout.setObjectName("gridLayout")
        self.label_username = QtWidgets.QLabel(dialog_authentication)
        self.label_username.setObjectName("label_username")
        self.gridLayout.addWidget(self.label_username, 0, 0, 1, 1)
        self.label_autologin_key = QtWidgets.QLabel(dialog_authentication)
        self.label_autologin_key.setObjectName("label_autologin_key")
        self.gridLayout.addWidget(self.label_autologin_key, 6, 0, 1, 1)
        self.label_password = QtWidgets.QLabel(dialog_authentication)
        self.label_password.setObjectName("label_password")
        self.gridLayout.addWidget(self.label_password, 1, 0, 1, 1)
        self.input_lineedit_autologin_key = QtWidgets.QLineEdit(dialog_authentication)
        self.input_lineedit_autologin_key.setObjectName("input_lineedit_autologin_key")
        self.gridLayout.addWidget(self.input_lineedit_autologin_key, 6, 1, 1, 1)
        self.input_lineedit_username = QtWidgets.QLineEdit(dialog_authentication)
        self.input_lineedit_username.setFrame(True)
        self.input_lineedit_username.setObjectName("input_lineedit_username")
        self.gridLayout.addWidget(self.input_lineedit_username, 0, 1, 1, 1)
        self.input_lineedit_password = QtWidgets.QLineEdit(dialog_authentication)
        self.input_lineedit_password.setMinimumSize(QtCore.QSize(200, 0))
        self.input_lineedit_password.setBaseSize(QtCore.QSize(0, 0))
        self.input_lineedit_password.setEchoMode(QtWidgets.QLineEdit.Password)
        self.input_lineedit_password.setObjectName("input_lineedit_password")
        self.gridLayout.addWidget(self.input_lineedit_password, 1, 1, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.button_box = QtWidgets.QDialogButtonBox(dialog_authentication)
        self.button_box.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.button_box.setCenterButtons(False)
        self.button_box.setObjectName("button_box")
        self.horizontalLayout.addWidget(self.button_box)
        self.gridLayout.addLayout(self.horizontalLayout, 7, 0, 1, 2)
        self.input_checkbox_use_autologin = QtWidgets.QCheckBox(dialog_authentication)
        self.input_checkbox_use_autologin.setObjectName("input_checkbox_use_autologin")
        self.gridLayout.addWidget(self.input_checkbox_use_autologin, 4, 0, 1, 2)
        self.input_checkbox_save_password = QtWidgets.QCheckBox(dialog_authentication)
        self.input_checkbox_save_password.setObjectName("input_checkbox_save_password")
        self.gridLayout.addWidget(self.input_checkbox_save_password, 3, 0, 1, 2)

        self.retranslateUi(dialog_authentication)
        QtCore.QMetaObject.connectSlotsByName(dialog_authentication)

    def retranslateUi(self, dialog_authentication):
        _translate = QtCore.QCoreApplication.translate
        dialog_authentication.setWindowTitle(_translate("dialog_authentication", "Authentication"))
        self.label_username.setText(_translate("dialog_authentication", "Username:"))
        self.label_autologin_key.setText(_translate("dialog_authentication", "Autologin key:"))
        self.label_password.setText(_translate("dialog_authentication", "Password:"))
        self.input_checkbox_use_autologin.setText(_translate("dialog_authentication", "Use autologin"))
        self.input_checkbox_save_password.setText(_translate("dialog_authentication", "Save password"))




#!/usr/bin/env python3
# encoding: utf-8

# Nagstamon - Nagios status monitor for your desktop
# Copyright (C) 2008-2021 Henri Wahl <henri@nagstamon.de> et al.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# from distutils.core import setup
import sys
import platform
import os.path

from Nagstamon.Config import AppInfo
from Nagstamon.Helpers import get_distro

# dummy debug queue for compiling
debug_queue = list()

NAME = AppInfo.NAME

OS = platform.system()
# make name lowercase for Linux/Unix
if OS not in ['Windows', 'Darwin']:
    if OS == 'Linux':
        DIST, DIST_VERSION, DIST_NAME = get_distro()
    # platform.dist() returns "('', '', '')" on FreeBSD
    elif OS == 'FreeBSD':
        DIST, DIST_VERSION, DIST_NAME = ('', '', '')
    # platform.dist() does not exist on NetBSD
    elif OS == 'NetBSD':
        DIST, DIST_VERSION, DIST_NAME = ('', '', '')
    else:
        DIST, DIST_VERSION, DIST_NAME = platform.dist()
    NAME = NAME.lower()
VERSION = AppInfo.VERSION.replace('-', '.') + '.' + DIST + DIST_VERSION

NAGSTAMON_SCRIPT = 'nagstamon.py'

from setuptools import setup
os_dependent_include_files = ['Nagstamon/resources']
executables = []
if os.path.exists('nagstamon'):
    NAGSTAMON_SCRIPT = 'nagstamon'

CLASSIFIERS = ['Intended Audience :: System Administrators',
               'Development Status :: 5 - Production/Stable',
               'Environment :: Win32 (MS Windows)',
               'Environment :: X11 Applications',
               'Environment :: MacOS X',
               'License :: OSI Approved :: GNU General Public License (GPL)',
               'Operating System :: Microsoft :: Windows',
               'Operating System :: POSIX :: Linux',
               'Operating System :: POSIX',
               'Natural Language :: English',
               'Programming Language :: Python',
               'Topic :: System :: Monitoring',
               'Topic :: System :: Networking :: Monitoring']

# Dependencies are automatically detected, but it might need
# fine tuning.
build_exe_options = dict(packages=['PyQt5.QtNetwork',
                                   'keyring.backends.kwallet',
                                   'keyring.backends.OS_X',
                                   'keyring.backends.SecretService',
                                   'keyring.backends.Windows'],
                         include_files=os_dependent_include_files,
                         include_msvcr=True,
                         excludes=[])

bdist_mac_options = dict(iconfile='Nagstamon/resources/nagstamon.icns',
                         custom_info_plist='Nagstamon/resources/Info.plist')

bdist_dmg_options = dict(volume_label='{0} {1}'.format(NAME, VERSION),
                         applications_shortcut=False)

bdist_rpm_options = dict(requires='python3 '
        'python3-beautifulsoup4 '
        'python3-crypto '
        'python3-cryptography '
        'python3-dateutil '
        'python3-keyring '
        'python3-lxml '
        'python3-psutil '
        'python3-pysocks '
        'python3-qt5 '
        'python3-requests '
        'python3-requests-kerberos '
        'python3-SecretStorage '
        'qt5-qtmultimedia '
        'qt5-qtsvg ',
dist_dir='./build')

setup(name=NAME,
      version=VERSION,
      license='GNU GPL v2',
      description='Nagios status monitor for desktop',
      long_description='Nagstamon is a Nagios status monitor which takes place in systray or on desktop (GNOME, KDE, Windows) as floating statusbar to inform you in realtime about the status of your Nagios and derivatives monitored network. It allows to connect to multiple Nagios, Icinga, Opsview, Op5Monitor, Checkmk/Multisite, Centreon and Thruk servers.',
      classifiers=CLASSIFIERS,
      author='Henri Wahl',
      author_email='henri@nagstamon.de',
      url='https://nagstamon.de',
      download_url='https://nagstamon.de/download',
      scripts=[NAGSTAMON_SCRIPT],
      packages=['Nagstamon',
                'Nagstamon.QUI',
                'Nagstamon.Servers',
                'Nagstamon.Servers.Alertmanager',
                'Nagstamon.thirdparty',
                'Nagstamon.thirdparty.Xlib',
                'Nagstamon.thirdparty.Xlib.ext',
                'Nagstamon.thirdparty.Xlib.protocol',
                'Nagstamon.thirdparty.Xlib.support',
                'Nagstamon.thirdparty.Xlib.xobject'],
      package_dir={'Nagstamon': 'Nagstamon'},
      package_data={'Nagstamon': ['resources/*']},
      data_files=[('%s/share/man/man1' % sys.prefix, ['Nagstamon/resources/nagstamon.1.gz']),
          ('%s/share/pixmaps' % sys.prefix, ['Nagstamon/resources/nagstamon.svg']),
          ('%s/share/applications' % sys.prefix, ['Nagstamon/resources/nagstamon.desktop'])],
      options=dict(build_exe=build_exe_options,
          bdist_mac=bdist_mac_options,
          bdist_dmg=bdist_dmg_options,
          bdist_rpm=bdist_rpm_options),
      executables=executables
      )



from Nagstamon.Objects import GenericService

class AlertmanagerService(GenericService):
    """
    add alertmanager specific service property to generic service class
    """
    service_object_id = ""
    labels = {}


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'dialog_downtime.ui'
#
# Created by: PyQt5 UI code generator 5.5.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_dialog_downtime(object):
    def setupUi(self, dialog_downtime):
        dialog_downtime.setObjectName("dialog_downtime")
        dialog_downtime.resize(409, 294)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(dialog_downtime.sizePolicy().hasHeightForWidth())
        dialog_downtime.setSizePolicy(sizePolicy)
        dialog_downtime.setSizeGripEnabled(True)
        dialog_downtime.setModal(True)
        self.gridLayout = QtWidgets.QGridLayout(dialog_downtime)
        self.gridLayout.setObjectName("gridLayout")
        self.label_start_time = QtWidgets.QLabel(dialog_downtime)
        self.label_start_time.setObjectName("label_start_time")
        self.gridLayout.addWidget(self.label_start_time, 2, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 13, 2, 1, 1)
        self.input_label_description = QtWidgets.QLabel(dialog_downtime)
        self.input_label_description.setObjectName("input_label_description")
        self.gridLayout.addWidget(self.input_label_description, 0, 0, 1, 5)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.button_change_defaults_downtime = QtWidgets.QPushButton(dialog_downtime)
        self.button_change_defaults_downtime.setObjectName("button_change_defaults_downtime")
        self.horizontalLayout.addWidget(self.button_change_defaults_downtime)
        self.button_box = QtWidgets.QDialogButtonBox(dialog_downtime)
        self.button_box.setOrientation(QtCore.Qt.Horizontal)
        self.button_box.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.button_box.setObjectName("button_box")
        self.horizontalLayout.addWidget(self.button_box)
        self.gridLayout.addLayout(self.horizontalLayout, 18, 0, 2, 5)
        self.label_duration_hours = QtWidgets.QLabel(dialog_downtime)
        self.label_duration_hours.setObjectName("label_duration_hours")
        self.gridLayout.addWidget(self.label_duration_hours, 12, 2, 1, 1)
        self.input_spinbox_duration_minutes = QtWidgets.QSpinBox(dialog_downtime)
        self.input_spinbox_duration_minutes.setObjectName("input_spinbox_duration_minutes")
        self.gridLayout.addWidget(self.input_spinbox_duration_minutes, 12, 3, 1, 1)
        self.input_lineedit_comment = QtWidgets.QLineEdit(dialog_downtime)
        self.input_lineedit_comment.setObjectName("input_lineedit_comment")
        self.gridLayout.addWidget(self.input_lineedit_comment, 1, 0, 1, 5)
        self.label_duration_minutes = QtWidgets.QLabel(dialog_downtime)
        self.label_duration_minutes.setObjectName("label_duration_minutes")
        self.gridLayout.addWidget(self.label_duration_minutes, 12, 4, 1, 1)
        self.input_spinbox_duration_hours = QtWidgets.QSpinBox(dialog_downtime)
        self.input_spinbox_duration_hours.setObjectName("input_spinbox_duration_hours")
        self.gridLayout.addWidget(self.input_spinbox_duration_hours, 12, 1, 1, 1)
        self.label_duration = QtWidgets.QLabel(dialog_downtime)
        self.label_duration.setObjectName("label_duration")
        self.gridLayout.addWidget(self.label_duration, 12, 0, 1, 1)
        self.label_end_time = QtWidgets.QLabel(dialog_downtime)
        self.label_end_time.setObjectName("label_end_time")
        self.gridLayout.addWidget(self.label_end_time, 3, 0, 1, 1)
        self.input_lineedit_end_time = QtWidgets.QLineEdit(dialog_downtime)
        self.input_lineedit_end_time.setObjectName("input_lineedit_end_time")
        self.gridLayout.addWidget(self.input_lineedit_end_time, 3, 1, 1, 4)
        self.input_lineedit_start_time = QtWidgets.QLineEdit(dialog_downtime)
        self.input_lineedit_start_time.setObjectName("input_lineedit_start_time")
        self.gridLayout.addWidget(self.input_lineedit_start_time, 2, 1, 1, 4)
        self.input_radiobutton_type_flexible = QtWidgets.QRadioButton(dialog_downtime)
        self.input_radiobutton_type_flexible.setObjectName("input_radiobutton_type_flexible")
        self.gridLayout.addWidget(self.input_radiobutton_type_flexible, 6, 0, 1, 1)
        self.input_radiobutton_type_fixed = QtWidgets.QRadioButton(dialog_downtime)
        self.input_radiobutton_type_fixed.setObjectName("input_radiobutton_type_fixed")
        self.gridLayout.addWidget(self.input_radiobutton_type_fixed, 5, 0, 1, 1)

        self.retranslateUi(dialog_downtime)
        self.button_box.accepted.connect(dialog_downtime.accept)
        self.button_box.rejected.connect(dialog_downtime.reject)
        QtCore.QMetaObject.connectSlotsByName(dialog_downtime)
        dialog_downtime.setTabOrder(self.input_lineedit_comment, self.input_spinbox_duration_hours)
        dialog_downtime.setTabOrder(self.input_spinbox_duration_hours, self.input_spinbox_duration_minutes)
        dialog_downtime.setTabOrder(self.input_spinbox_duration_minutes, self.button_change_defaults_downtime)

    def retranslateUi(self, dialog_downtime):
        _translate = QtCore.QCoreApplication.translate
        dialog_downtime.setWindowTitle(_translate("dialog_downtime", "Downtime"))
        self.label_start_time.setText(_translate("dialog_downtime", "Start time:"))
        self.input_label_description.setText(_translate("dialog_downtime", "description - set by QUI.py"))
        self.button_change_defaults_downtime.setText(_translate("dialog_downtime", "Change downtime defaults..."))
        self.label_duration_hours.setText(_translate("dialog_downtime", "hours"))
        self.label_duration_minutes.setText(_translate("dialog_downtime", "minutes"))
        self.label_duration.setText(_translate("dialog_downtime", "Duration:"))
        self.label_end_time.setText(_translate("dialog_downtime", "End time:"))
        self.input_lineedit_end_time.setText(_translate("dialog_downtime", "n/a"))
        self.input_lineedit_start_time.setText(_translate("dialog_downtime", "n/a"))
        self.input_radiobutton_type_flexible.setText(_translate("dialog_downtime", "Flexible"))
        self.input_radiobutton_type_fixed.setText(_translate("dialog_downtime", "Fixed"))

